from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
import json
import uvicorn
from spai.storage import Storage

app = FastAPI(title="analytics")
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)


@app.get("/")
async def analytics():
    try:
        storage = Storage("data")
        analytics = storage.read("stats.csv")
        return json.loads(analytics.to_json())
    except Exception as e:
        return {}


# need this to run in background
if __name__ == "__main__":
    uvicorn.run(app, host="0.0.0.0", port=8000)



from pydantic import BaseModel
from typing import List


class GetServices:
    def __init__(self, repo):
        self.repo = repo

    class Inputs(BaseModel):
        user: dict
        project: str

    class Outputs(BaseModel):
        services: List[str]

    def __call__(self, inputs: Inputs) -> Outputs:
        response = self.repo.retrieve_project(inputs.user, inputs.project)
        if response.status_code == 200:
            project = response.json()
            services = project["services"]
            names = [service["name"] for service in services]
            return self.Outputs(services=names)
        raise Exception(response.json()["detail"])



class ResolutionError(Exception):
    def __init__(self, resolution, size):
        self.message = f"Image shape at {resolution} m resolution: {size} pixels. Currently, we only support size smaller than 2500 x 2500 pixels."
        super().__init__(self.message)


class ImageOutOfBounds(Exception):
    def __init__(self, x, y, z):
        self.message = f"Tile {x}/{y}/{z} is out of bounds"
        super().__init__(self.message)


class ImageDoesNotExistError(Exception):
    message = "Image doesn't exist"

    def __init__(self):
        super().__init__(self.message)



import os
import json

from ..project import get_services, stop_service, run_service, schedule_service


def setup_files(item_type, dir):
    files = {
        item_type: open(
            dir / f'main.{"ipynb" if item_type == "notebook" else "py"}', "rb"
        ),
        "requirements": open(dir / "requirements.txt", "rb")
        if "requirements.txt" in os.listdir(dir)
        else None,
        "env": open(dir / ".env", "rb") if ".env" in os.listdir(dir) else None,
    }
    return files


def run_item(user, item, dir, typer, config, rebuild, item_type):
    item_dir = dir / item.name
    typer.echo(f"Running item '{item.name}'...")
    data = {
        "name": item.name,
        "project": config.project,
        "rebuild": rebuild,
        "command": item.command,
    }
    if item_type == "api" or item_type == "ui":
        data["port"] = item.port
    if item_type == "ui":
        data["command"] = item.command
        if item.env:
            data["env_vars"] = json.dumps(item.env)
    if item_type != "ui":
        data["storage"] = item.storage
    files = setup_files(item_type, item_dir)
    return run_service(user, item_type, files, data)


def schedule_item(user, item, dir, typer, config, rebuild, item_type):
    item_dir = dir / item.name
    typer.echo(f"Setting up cronjob for item '{item.name}'...")
    files = setup_files(item_type, item_dir)
    data = {
        "name": item.name,
        "project": config.project,
        "run_every": item.run_every,
        "storage": item.storage,
        "rebuild": rebuild,
    }
    return schedule_service(user, item_type, files, data)


def run_script_or_notebook(user, item, dir, typer, config, rebuild, item_type):
    if item.run_on_start:
        return run_item(user, item, dir, typer, config, rebuild, item_type)
    if item.run_every:
        return schedule_item(user, item, dir, typer, config, rebuild, item_type)


def check_item(user, item, project, services, typer, item_type):
    name = f"{item_type}.{item.name}"
    if name in services:
        typer.echo(f"Service '{name}' already deployed.")
        typer.echo(f"Stopping ...")  # confirm?
        stop_service(user, project, item_type, item.name)


def deploy_cloud(user, dir, config, typer, rebuild):
    typer.echo(f"Deploying...")
    try:
        services = get_services(user, config.project)
    except Exception as e:
        services = []  # project not found
    if config.scripts:
        typer.echo(f"Deploying scripts...")
        for script in config.scripts:
            check_item(user, script, config.project, services, typer, "script")
            message = run_script_or_notebook(
                user, script, dir / "scripts", typer, config, rebuild, "script"
            )
            typer.echo(message)
    if config.notebooks:
        typer.echo(f"Deploying notebooks...")
        for notebook in config.notebooks:
            check_item(user, notebook, config.project, services, typer, "notebook")
            run_script_or_notebook(
                user, notebook, dir / "notebooks", typer, config, rebuild, "notebook"
            )
    if config.apis:
        typer.echo(f"Deploying APIs...")
        for api in config.apis:
            check_item(user, api, config.project, services, typer, "api")
            run_item(user, api, dir / "apis", typer, config, rebuild, "api")
    if config.uis:
        typer.echo(f"Deploying UIs...")
        for ui in config.uis:
            check_item(user, ui, config.project, services, typer, "ui")
            run_item(user, ui, dir / "uis", typer, config, rebuild, "ui")
    # TODO: delete running services that no longer are in config???



from minio import Minio
import os


def get_client():
    if "S3_REGION" in os.environ:
        return Minio(
            endpoint=os.environ["S3_ENDPOINT"],
            access_key=os.environ["ACCESS_KEY_ID"],
            secret_key=os.environ["SECRET_ACCESS_KEY"],
            secure=True,
            region=os.environ["S3_REGION"],
        )
    return Minio(
        endpoint=os.environ["S3_ENDPOINT"],
        access_key=os.environ["ACCESS_KEY_ID"],
        secret_key=os.environ["SECRET_ACCESS_KEY"],
        secure=False,
    )



from pydantic import BaseModel
import time 

from ...errors.auth import LoginError, AuthTimeOut

class Auth:
    def __init__(self, repo, api_repo, max_t=30, interval=2):
        self.repo = repo
        self.api_repo = api_repo
        self.max_t = max_t
        self.interval = interval

    class Inputs(BaseModel):
        pass 

    class Outputs(BaseModel):
        user: dict = None
        token: str = None

    def __call__(self, inputs: Inputs) -> Outputs:
        response = self.api_repo.login()
        if response.status_code != 200:
            raise LoginError()
        data = response.json()
        print('On your computer or mobile device navigate to: ', data['login_url'])
        authenticated = False
        t0 = time.time()
        while not authenticated and time.time() - t0 < self.max_t:
            response = self.api_repo.token(data['code'])
            token_data = response.json()
            if response.status_code == 200:
                print('Authenticated!')
                print('- Id Token: {}...'.format(token_data['id_token'][:10]))
                # save token data in file
                creds_path = self.repo.save_creds(token_data)
                print('Saved credentials to: ', creds_path)
                current_user = self.repo.decode_token(token_data)
                authenticated = True
                current_user['id_token'] = token_data['id_token']
                return self.Outputs(user=current_user)
            else:
                time.sleep(self.interval)
        if not authenticated:
            raise AuthTimeOut()


import os
from glob import glob
import rasterio as rio
import numpy as np
import pandas as pd
import shutil

from .BaseStorage import BaseStorage


class Storage(BaseStorage):
    def __init__(self, path="data"):
        super().__init__()
        self.path = path
        if not os.path.exists(path):
            os.makedirs(path)
            print(f"{path} created")

    def get_path(self, name):
        return os.path.join(self.path, name)

    def create_from_path(self, data, name):
        dst_path = self.get_path(name)
        shutil.move(data, dst_path)
        return dst_path

    def create_from_image(self, data, name):
        dst_path = self.get_path(name)
        data.save(dst_path)
        return dst_path

    def create_from_rasterio(self, x, name, ds, window=None):
        dst_path = self.get_path(name)
        kwargs = ds.meta.copy()
        transform = ds.transform if window is None else ds.window_transform(window)
        kwargs.update(
            driver="GTiff",
            count=1 if x.ndim < 3 else x.shape[0],
            height=x.shape[0] if x.ndim < 3 else x.shape[1],
            width=x.shape[1] if x.ndim < 3 else x.shape[2],
            dtype=np.uint8 if x.dtype == "bool" else x.dtype,
            crs=ds.crs,
            transform=transform,
            # nbits=1 if x.dtype == 'bool' else
        )
        with rio.open(dst_path, "w", **kwargs) as dst:
            bands = 1 if x.ndim < 3 else [i + 1 for i in range(x.shape[0])]
            dst.write(x, bands)
        return dst_path

    def create_from_array(self, data, name):
        dst_path = self.get_path(name)
        np.save(dst_path, data)
        return dst_path

    def create_from_csv(self, data, name):
        dst_path = self.get_path(name)
        data.to_csv(dst_path)
        return dst_path

    def create_from_json(self, data, name):
        dst_path = self.get_path(name)
        data.to_json(dst_path)
        return dst_path

    def list(self, pattern="*"):
        paths = glob(os.path.join(self.path, pattern))
        # strip base path
        return [p.replace(self.path + "/", "") for p in paths]

    def read_from_array(self, name, path=None):
        if path is None:
            path = self.get_path(name)
        return np.load(path)

    def read_from_rasterio(self, name):
        return rio.open(self.get_path(name))

    def read_from_csv(self, name):
        return pd.read_csv(self.get_path(name), index_col=0)

    def read_from_json(self, name):
        return pd.read_json(self.get_path(name))



import os
import geopandas as gpd
import overpy
import geojson
from shapely.geometry import box
from shapely.geometry.polygon import Polygon
from tqdm import tqdm


def validate_coords(coords):
    # Coordinates of points of a polygon: [[lon1,lat1],[lon2,lat2], ... ,[lonN,latN]]
    for coord in coords:
        # check that each coord is a list of 2 coordinates
        if len(coord) != 2:
            raise Exception("each coord must be a list of 2 coordinates")
        # check that each coordinate is a float
        if not isinstance(coord[0], float) or not isinstance(coord[1], float):
            raise Exception("each coordinate must be a float")
        lon = coord[0]
        lat = coord[1]

        # check lat and long ranges
        if lon < -180 or lon > 180 or lat < -90 or lat > 90:
            raise Exception("each coordinate must be a valid lat/long")
    return coords


def validate_bounds(bounds):
    # check that bounds is a list
    if len(bounds) != 4:
        raise Exception(
            "bounds must be a tuple of 4 points: (minlon, minlat, maxlon, maxlat))"
        )
    # check that each bound is a float
    for bound in bounds:
        if not isinstance(bound, float):
            raise Exception("each bound must be a float")
    # check lat and long ranges
    minlon = bounds[0]
    minlat = bounds[1]
    maxlon = bounds[2]
    maxlat = bounds[3]

    if minlat < -90 or minlat > 90 or maxlat < -90 or maxlat > 90:
        raise Exception("each latitude must be a valid lat/long")
    if minlon < -180 or minlon > 180 or maxlon < -180 or maxlon > 180:
        raise Exception("each longitude must be a valid lat/long")
    # check that minlat < maxlat and minlng < maxlng
    if minlon > maxlon or minlat > maxlat:
        raise Exception(
            "minlat must be less than maxlat and minlng must be less than maxlng"
        )
    return bounds


def create_aoi_geodataframe(obj):
    # obj can be a :
    # GeoDataFrame,
    # GeoJSON file,
    # Path to a GeoJSON file,
    # Polygon of Shapely (Bounding Box),
    # List of Coords of a (Bounding Box),
    # Location name (string)

    # return a GeoDataFrame of the Bounding Box of the obj or a list of gdfs if obj is a location name and there are more than one relation

    if (
        isinstance(obj, str)
        and not os.path.isfile(obj)
        and not obj.endswith(".geojson")
    ):
        # case 1: obj is a location name (e.g. "Madrid")
        obj = get_bbox_from_location_name(obj)  # return Box of Shapely or list of gdfs
        if isinstance(obj, list):
            return obj

    elif isinstance(obj, list):
        # case 2: obj is a list of coords of a Polygon: [[lat1,long1],[lat2,long2],...,[latN,longN]]
        coords = validate_coords(obj)
        poly = Polygon(coords)
        bounds = poly.bounds
        obj = box(*bounds)  # return Box of Shapely

    if isinstance(obj, tuple):
        # case 3: obj is a tuple of bounds: (minlat, minlng, maxlat, maxlng)
        bounds = validate_bounds(obj)
        obj = box(*bounds)  # return Box of Shapely

    if isinstance(obj, Polygon):
        # case 4: obj is a Box of Shapely: Polygon[[lat1,long1],[lat2,long2],[lat3,long3],[lat4,long4]]
        bounds = obj.bounds
        validate_bounds(bounds)
        return gpd.GeoDataFrame(geometry=[obj], crs=4326)

    if isinstance(obj, gpd.GeoDataFrame):
        # case 5: obj is a GeoDataFrame
        return obj.to_crs(4326)

    if (
        isinstance(obj, dict)
        and obj.get("type") == "FeatureCollection"
        and "features" in obj
    ):
        # case 6: obj is a GeoJSON file
        if obj["type"] == "Polygon":
            for coords in obj["coordinates"][0]:
                validate_coords(coords)
        elif obj["type"] == "MultiPolygon":
            for _coords in obj["coordinates"][0]:
                for coords in _coords:
                    validate_coords(coords)
        return gpd.GeoDataFrame.from_features(obj, crs=4326)

    if isinstance(obj, str) and os.path.isfile(obj) and obj.endswith(".geojson"):
        # case 7: obj is a path to GeoJSON file
        geojson_file = geojson.load(open(obj))
        if geojson_file["type"] == "Polygon":
            for coords in geojson_file["coordinates"][0]:
                validate_coords(coords)
        elif geojson_file["type"] == "MultiPolygon":
            for _coords in geojson_file["coordinates"][0]:
                for coords in _coords:
                    validate_coords(coords)
        return gpd.GeoDataFrame.from_features(geojson_file, crs=4326)

    raise Exception(f"location {obj} not supported")


def get_bbox_from_relation(relation):
    api = overpy.Overpass()  # Create an Overpass API instance
    minlat, minlon, maxlat, maxlon = None, None, None, None
    # En Barcelona esto es muuuy lento...
    for member in relation.members:
        if member.role == "outer":
            way_id = member.ref
            way_result = api.query(
                f"way({way_id});(._;>;);out;"
            )  # Retrieve the way and its nodes in a single query

            if way_result.ways:
                way = way_result.ways[0]
                for node in way.nodes:
                    lat, lon = float(node.lat), float(node.lon)

                    if minlat is None or lat < minlat:
                        minlat = lat
                    if minlon is None or lon < minlon:
                        minlon = lon
                    if maxlat is None or lat > maxlat:
                        maxlat = lat
                    if maxlon is None or lon > maxlon:
                        maxlon = lon
    validate_bounds([minlon, minlat, maxlon, maxlat])
    return box(minlon, minlat, maxlon, maxlat)


def get_bbox_from_location_name(location_name):
    location_name = capitalize_location(location_name)
    api = overpy.Overpass()  # Create an Overpass API instance

    query = f"""
    (
    relation["type"="boundary"]["name"="{location_name}"];
    );
    out body;
    """
    result = api.query(query)  # Send the query to Overpass API
    print(result)
    gdfs = []
    if len(result.relations) > 1:
        # if there are more than one relation, return a dict of gdfs
        for relation in tqdm(result.relations):
            name = relation.tags.get("wikipedia")
            bbox = get_bbox_from_relation(relation)
            gdf = gpd.GeoDataFrame(geometry=[bbox], crs=4326)  # validate crs
            gdfs.append({"name": name, "gdf": gdf})
        return gdfs
    else:
        if result.relations:
            # if there is only one relation, return a Shapely Bounding Box
            relation = result.relations[0]
            return get_bbox_from_relation(relation)
        else:
            # if there is no relation, raise an exception
            raise Exception(f"location {location_name} not found")


def capitalize_location(location_name):
    # capitalize each word of the location name (e.g. "madrid" -> "Madrid")
    return " ".join(
        [
            word.capitalize() if len(word) > 3 else word.lower()
            for word in location_name.split(" ")
        ]
    )



import os
import schedule
import time
from multiprocessing import Process


def run_item(item, command):
    if item.command:
        os.system(item.command)
    else:
        os.system(command)


def run_script(args):
    script, dir, typer = args
    typer.echo(f"Running script '{script.name}'...")
    os.chdir(dir)  # change to project dir
    run_item(script, f"python scripts/{script.name}/main.py")


def run_notebook(args):
    notebook, dir, typer = args
    typer.echo(f"Running notebook '{notebook.name}'...")
    os.chdir(dir / "notebooks" / notebook.name)  # necessary ???
    run_item(
        notebook,
        f"papermill main.ipynb output.ipynb",
    )
    # papermill can execute notebooks and save output in S3
    # papermill can execute notebooks with params


def run_api(api, dir, port, host):
    os.chdir(dir)
    os.system(f"python apis/{api.name}/main.py --port {port} --host {host}")


def run_ui(ui, dir, api_urls):
    os.chdir(dir)
    cmd = ""
    for name, value in ui.env.items():
        if value in api_urls:
            cmd += f"export {name}={api_urls[value]}; "
    cmd += ui.command
    os.system(cmd)


def run_schdule():
    while True:
        schedule.run_pending()
        time.sleep(1)


def run_local(dir, config, typer):
    typer.echo(f"Deploying locally...")
    keep_alive = False
    processes = []
    if config.scripts:
        for script in config.scripts:
            # the order affects in the execution of the scheduled tasks
            typer.echo(f"Deploying scripts...")
            # # TODO: reqs, env
            if script.run_on_start:
                run_script((script, dir, typer))
            if script.run_every:
                keep_alive = True
                schedule.every(script.run_every).minutes.do(
                    run_script, (script, dir, typer)
                )  # if a task goes after another than takes more time than the scheduled time, it will have to wait
    if config.notebooks:
        typer.echo(f"Deploying notebooks...")
        for notebook in config.notebooks:
            # TODO: reqs, env
            if notebook.run_on_start:
                run_notebook((notebook, dir, typer))
            if notebook.run_every:
                keep_alive = True
                schedule.every(notebook.run_every).minutes.do(
                    run_notebook, (notebook, dir, typer)
                )
    api_urls = {}
    if config.apis:
        typer.echo(f"Deploying apis...")
        for api in config.apis:
            typer.echo(f"Running api '{api.name}'...")
            # TODO: reqs, env
            # load_dotenv(f"{dir}/apis/{api.name}/.env")
            p = Process(target=run_api, args=(api, dir, api.port, api.host))
            p.start()
            processes.append(p)
            # save api_urls in case UIs need them
            api_urls[f"api.{api.name}"] = f"http://{api.host}:{api.port}"
    if config.uis:
        typer.echo(f"Deploying uis...")
        for ui in config.uis:
            typer.echo(f"Running ui '{ui.name}'...")
            # TODO: reqs, env
            # load_dotenv(f"{dir}/apis/{api.name}/.env")
            p = Process(target=run_ui, args=(ui, dir, api_urls))
            p.start()
            processes.append(p)
    typer.echo(f"Project '{config.project}' deployed.")
    if keep_alive:
        p = Process(target=run_schdule)
        p.start()
        processes.append(p)
    for p in processes:
        p.join()
    return



from pydantic import BaseModel


class ScheduleService:
    def __init__(self, repo):
        self.repo = repo

    class Inputs(BaseModel):
        user: dict
        service: str
        files: dict
        data: dict

    class Outputs(BaseModel):
        message: str

    def __call__(self, inputs: Inputs) -> Outputs:
        message = self.repo.schedule_service(
            inputs.user, inputs.service, inputs.files, inputs.data
        )
        return self.Outputs(message=message)



from spai.storage import Storage
import numpy as np

# retrieve all images
storage = Storage("data")
images = storage.list("S2*")
print("Found the following images:", images)


# compute ndvis


def ndvi(image):
    ds = storage.read(image)
    red = ds.read(4).astype(np.float32)
    nir = ds.read(8)
    ndvi = (nir - red) / (nir + red)
    date = image.split("_")[1].split(".")[0]
    return storage.create(ndvi, f"NDVI_{date}.tif", ds=ds)


paths = [ndvi(image) for image in images]
print("NDVI images saved at", paths)



from .GetServices import GetServices
from .StopService import StopService
from .RunService import RunService
from .ScheduleService import ScheduleService
from .GetLogs import GetLogs

from ...repos import APIRepo


def get_services(user, project):
    repo = APIRepo()
    services = GetServices(repo)
    inputs = services.Inputs(user=user, project=project)
    outputs = services(inputs)
    return outputs.services


def stop_service(user, project, service, name):
    repo = APIRepo()
    stop = StopService(repo)
    inputs = stop.Inputs(user=user, project=project, service=service, name=name)
    outputs = stop(inputs)
    return outputs.message


def run_service(user, service, files, data):
    repo = APIRepo()
    run = RunService(repo)
    inputs = run.Inputs(user=user, service=service, files=files, data=data)
    outputs = run(inputs)
    return outputs.message


def schedule_service(user, service, files, data):
    repo = APIRepo()
    schedule = ScheduleService(repo)
    inputs = schedule.Inputs(user=user, service=service, files=files, data=data)
    outputs = schedule(inputs)
    return outputs.message


def get_logs(user, project, service, name):
    repo = APIRepo()
    logs = GetLogs(repo)
    inputs = logs.Inputs(user=user, project=project, service=service, name=name)
    outputs = logs(inputs)
    return outputs.logs



import requests
import os


class APIRepo:
    # def __init__(self, url=os.getenv("SPAI_API_URL", "https://spai.api.dev.earthpulse.ai/")):
    def __init__(self, url=os.getenv("SPAI_API_URL", "http://localhost:8000/")):
        self.url = url

    def login(self):
        return requests.get(self.url + "auth/login")

    def token(self, code):
        return requests.get(self.url + "auth/token?code=" + code)

    def logout_url(self):
        response = requests.get(self.url + "auth/logout")
        return response.json()["logout_url"]

    def get_headers(self, user):
        return {"Authorization": "Bearer " + user["id_token"]}

    def retrieve_projects(self, user):
        return requests.get(
            self.url + "projects", headers=self.get_headers(user)
        ).json()

    def retrieve_project(self, user, project):
        return requests.get(
            self.url + f"projects/{project}", headers=self.get_headers(user)
        )

    def run_service(self, user, service, files, data):
        response = requests.post(
            self.url + f"run/{service}",
            data=data,
            files=files,
            headers=self.get_headers(user),
        )
        if response.status_code == 200:
            return response.text
        return "Something went wrong.\n" + response.json()["detail"]

    def schedule_service(self, user, service, files, data):
        response = requests.post(
            self.url + f"schedule/{service}",
            data=data,
            files=files,
            headers=self.get_headers(user),
        )
        if response.status_code == 200:
            return response.text
        return "Something went wrong.\n" + response.json()["detail"]

    def stop_service(self, user, project, service, name):
        response = requests.post(
            self.url + f"stop/{service}",
            json={"project": project, "name": name},
            headers=self.get_headers(user),
        )
        if response.status_code == 200:
            return f"Stopped."
        return "Something went wrong.\n" + response.json()["detail"]

    def get_logs(self, user, project, service, name):
        response = requests.post(
            self.url + f"logs/{service}",
            json={"project": project, "name": name},
            headers=self.get_headers(user),
        )
        if response.status_code == 200:
            return response.json()
        return "Something went wrong.\n" + response.json()["detail"]



import os
import yaml
from pydantic import BaseModel
from typing import Union, List


class ScriptConfig(BaseModel):
    name: str
    command: Union[str, None] = None
    run_every: Union[int, None] = None  # seconds (in cloud minutes)
    run_on_start: bool = True
    storage: Union[str, None] = None  # folder to bind in cloud


class NotebookConfig(BaseModel):
    name: str
    command: Union[str, None] = None
    run_every: Union[int, None] = None  # seconds
    run_on_start: bool = True
    storage: Union[str, None] = None  # folder to bind in cloud


class APIConfig(BaseModel):
    name: str
    command: Union[str, None] = None
    port: int = 8000
    host: str = "0.0.0.0"
    storage: Union[str, None] = None  # folder to bind in cloud


class UIConfig(BaseModel):
    name: str
    command: str  # steamlit, javascript, ...
    port: int = 3000
    env: dict = {}  # can accept the name of another service as a url placeholder


class Config(BaseModel):
    project: Union[str, None] = None
    scripts: Union[List[ScriptConfig], None] = None
    notebooks: Union[List[NotebookConfig], None] = None
    apis: Union[List[APIConfig], None] = None
    uis: Union[List[UIConfig], None] = None


def validate_folder(dir, folder, typer):
    if not os.path.exists(dir / folder):
        raise typer.BadParameter(f"No {folder} directory found in '{dir}'.")


def validate_item(dir, folder, item, name, typer, file="main.py"):
    # check name
    if not item.name:
        raise typer.BadParameter(f"{name} '{item.name}' is missing 'name' attribute.")
    # check folder has folder with item name
    if not os.path.exists(dir / folder / item.name):
        raise typer.BadParameter(
            f"{name} '{item.name}' cannot be found in {dir}/{folder}."
        )
    # check folder has file
    if not file in os.listdir(dir / folder / item.name):
        raise typer.BadParameter(f"{name} '{item.name}' is missing file 'main.py'.")
    # TODO: check optionals: reqs, env...


def load_and_validate_config(dir, typer, cloud=False):
    # check dir exists
    if not dir.exists():
        raise typer.BadParameter(f"Directory '{dir}' does not exist.")
    # check dir is a spai project
    if not "spai.config.yml" in os.listdir(dir):
        raise typer.BadParameter(
            f"Directory '{dir}' is not a spai project. No spai.config.yml file found."
        )
    # load config
    config = {}
    with open(dir / "spai.config.yml", "r") as f:
        config = yaml.safe_load(f)
    if not config:
        raise typer.BadParameter(f"spai.config.yml file is empty.")
    config = Config(**config)
    # TODO: check if project name is already taken in cloud, locally is not a problem
    config.project = dir.name if not config.project else config.project
    # check scripts
    if config.scripts:
        # check project has scripts folder
        validate_folder(dir, "scripts", typer)
        for script in config.scripts:
            validate_item(dir, "scripts", script, "script", typer)
    else:
        typer.echo("No scripts found in spai.config.yml.")
    # check notebooks
    if config.notebooks:
        # check project has notebooks folder
        validate_folder(dir, "notebooks", typer)
        for notebook in config.notebooks:
            validate_item(dir, "notebooks", notebook, "notebook", typer, "main.ipynb")
    # check apis
    if config.apis:
        # check project has apis folder
        validate_folder(dir, "apis", typer)
        for api in config.apis:
            validate_item(dir, "apis", api, "api", typer)
    # check uis
    if config.uis:
        # check project has uis folder
        validate_folder(dir, "uis", typer)
        for ui in config.uis:
            validate_item(dir, "uis", ui, "ui", typer)
    return config



from .sentinelhub import SHExplorer
from .utils import create_aoi_geodataframe


def explore_satellite_images(aoi, time_interval, sensor="S2L2A", **kargs):
    gdf = create_aoi_geodataframe(aoi)
    if isinstance(gdf, list):
        return gdf
    explorer = SHExplorer(time_interval, sensor, **kargs)
    results = explorer.search(gdf)
    return results



import shutil
from pathlib import Path
import os


def init_project(project):
    # ask for project name
    # copy template
    template = Path(__file__).parent / "project-template"
    shutil.copytree(template, Path(project))
    # change name to project in spai.config.yaml
    config = Path(project) / "spai.config.yml"
    os.system(f"sed -i 's/project-template/{project}/g' {config}")



import typer
import os
import sys
from pathlib import Path

# Add the cli directory to the Python path
spai_cli_dir = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(spai_cli_dir))

from cli.commands import auth as _auth

from cli.src.usecases.auth import auth
from cli.src.usecases.project import init_project, get_services, stop_service, get_logs
from cli.src.usecases.run import (
    run_local,
    deploy_cloud,
    load_and_validate_config,
)

app = typer.Typer()
app.add_typer(_auth.app, name="auth")


@app.command()
def init():
    project = typer.prompt("Project name")
    init_project(project)
    typer.echo(f"Project {project} created")


@app.command()
def list(
    project: str,
):
    try:
        user = auth()
        services = get_services(user, project)
        if len(services) == 0:
            return typer.echo(f"No services running in project '{project}'.")
        return typer.echo(f"Services in project '{project}': {services}")
    except Exception as e:
        return typer.echo(e)


@app.command()
def run(
    dir: Path,
):
    dir = Path(dir).resolve()
    config = load_and_validate_config(dir, typer)
    return run_local(dir, config, typer)


@app.command()
def deploy(
    dir: Path,
    rebuild: bool = typer.Option(False, "-r", "--rebuild"),  # force rebuild image
):
    dir = Path(dir).resolve()
    config = load_and_validate_config(dir, typer, True)
    user = auth()
    return deploy_cloud(user, dir, config, typer, rebuild)


@app.command()
def stop(
    project: str,
):
    try:
        user = auth()
        services = get_services(user, project)
        if len(services) == 0:
            return typer.echo(f"No services running in project '{project}'.")
        for service in services:
            service_type, name = service.split(".")
            typer.echo(f"Stopping service '{name}'...")
            stop_service(user, project, service_type, name)
        return typer.echo(f"Stopped all scripts in project '{project}'.")
    except Exception as e:
        return typer.echo(e)


@app.command()
def logs(
    project: str,
    service: str,
):
    service, name = service.split(".")
    user = auth()
    logs = get_logs(user, project, service, name)
    typer.echo(logs)


if __name__ == "__main__":
    app()



from pydantic import BaseModel
from typing import List


class StopService:
    def __init__(self, repo):
        self.repo = repo

    class Inputs(BaseModel):
        user: dict
        project: str
        service: str
        name: str

    class Outputs(BaseModel):
        message: str

    def __call__(self, inputs: Inputs) -> Outputs:
        message = self.repo.stop_service(
            inputs.user, inputs.project, inputs.service, inputs.name
        )
        return self.Outputs(message=message)



from .sentinelhub import SHS2L2ADownloader, SHS2L1CDownloader, SHS1Downloader
from .utils import create_aoi_geodataframe
import shutil
import os
import geopandas as gpd

def download_satellite_image(aoi, date, sensor, storage):
    gdf = create_aoi_geodataframe(aoi)
    if isinstance(gdf, gpd.GeoDataFrame):
        download_dir = "/tmp/sentinelhub"
        if sensor == "S2L2A":
            downloader = SHS2L2ADownloader(download_dir)
        elif sensor == "S2L1C":
            downloader = SHS2L1CDownloader(download_dir)
        elif sensor == "S1":
            downloader = SHS1Downloader(download_dir)
        else:
            raise Exception(f"sensor {sensor} not supported")
        dst_path = downloader.download(gdf, date)
        dst_path = storage.create(dst_path, name=f"{sensor}_{date}.tif")
        shutil.rmtree(download_dir)
        return dst_path
    else:
        return gdf


from ...repos import AuthRepo, APIRepo
from .IsLogged import IsLogged
from .Auth import Auth
from .Logout import Logout


def is_logged():
    repo = AuthRepo()
    _is_logged = IsLogged(repo)
    inputs = _is_logged.Inputs()
    outputs = _is_logged(inputs)
    return outputs.user


def auth():
    user = is_logged()
    if user:
        return user
    api_repo = APIRepo()
    repo = AuthRepo()
    _auth = Auth(repo, api_repo)
    inputs = _auth.Inputs()
    outputs = _auth(inputs)
    return outputs.user


def generate_logout_url():
    api_repo = APIRepo()
    repo = AuthRepo()
    _logout = Logout(repo, api_repo)
    inputs = _logout.Inputs()
    outputs = _logout(inputs)
    return outputs.logout_url



import typer
from ..src.usecases.auth import is_logged, auth, generate_logout_url
from ..src.errors.auth import LoginError

app = typer.Typer()


@app.command()
def login():
    """
    Login to your account
    """
    try:
        user = auth()
        typer.echo(f"You are logged in as {user['email']}")
    except LoginError as e:
        typer.echo(e.message)
        raise typer.Exit(code=1)


@app.command()
def logout():
    """
    Logout from your account
    """
    user = is_logged()
    if user:
        typer.echo(f"You are logged in as {user['email']}")
        typer.confirm(f"Are you sure you want to logout?", abort=True)
        logout_url = generate_logout_url()
        typer.echo(f"You are logged out.")
        typer.echo(
            f"If you want to login with a different account, visit {logout_url} and login again."
        )
    else:
        typer.echo(f"You are not logged in.")


if __name__ == "__main__":
    app()
